import 'dart:async';

import 'package:apus_drones_mobile/models/AddressModel.dart';
import 'package:apus_drones_mobile/services/LocationService.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:places_service/places_service.dart';

typedef AddressChanged = void Function(AddressModel address);

class AddressAutocomplete extends StatefulWidget {
  final String placeholder;
  final AddressChanged? onAddressSelected;
  String? errorText;
  AddressModel? selectedAddress;
  ScrollController? scrollController;

  AddressAutocomplete({
    Key? key,
    this.onAddressSelected,
    this.placeholder = "",
    this.errorText,
    this.scrollController,
  }) : super(key: key);

  @override
  _AddressAutocompleteState createState() => _AddressAutocompleteState();
}

class _AddressAutocompleteState extends State<AddressAutocomplete> {
  Timer? _debounce;
  LocationService locationService = LocationService();
  List<PlacesAutoCompleteResult> options = [];
  TextEditingController inputController = TextEditingController();

  _onSearchChanged(String query) {
    final _scrollController = widget.scrollController;
    if (_debounce?.isActive ?? false) {
      _debounce?.cancel();
    }
    _debounce = Timer(const Duration(milliseconds: 500), () async {
      final places = await locationService.getPlaces(query);
      setState(() {
        this.options = places;
      });
      if (this.options.length > 0) {
        _scrollController?.animateTo(
          300,
          duration: Duration(milliseconds: 200),
          curve: Curves.fastOutSlowIn,
        );
      }
    });
  }

  @override
  void dispose() {
    _debounce?.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Column(children: [
      TextField(
        decoration: InputDecoration(
          labelText: this.widget.placeholder,
          errorText: this.widget.errorText,
          border: OutlineInputBorder(),
        ),
        onChanged: this._onSearchChanged,
        controller: inputController,
      ),
      ...this.options.map(
            (option) => ListTile(
              onTap: () async {
                final PlacesDetails? addressDetails =
                    await locationService.getPlaceDetails(option.placeId ?? "");

                if (addressDetails == null) {
                  return;
                }

                this.widget.selectedAddress = AddressModel(
                    id: 0,
                    description: addressDetails.streetLong.toString(),
                    number: int.parse(addressDetails.streetNumber!.isNotEmpty
                        ? addressDetails.streetNumber.toString()
                        : "-1"),
                    lat: addressDetails.lat!,
                    lng: addressDetails.lng!,
                    zipCode: addressDetails.zip!);

                this.inputController.text = option.description!;

                FocusScope.of(context).unfocus();

                if (this.widget.onAddressSelected != null) {
                  this.widget.onAddressSelected!(this.widget.selectedAddress!);
                }

                setState(() {
                  this.options = [];
                });
              },
              title: Text(option.description!),
            ),
          ),
    ]);
  }
}
