import 'dart:async';

import 'package:apus_drones_mobile/constants/OrderStatus.dart';
import 'package:apus_drones_mobile/models/OrderItemModel.dart';
import 'package:apus_drones_mobile/models/PartnerOrderModel.dart';
import 'package:apus_drones_mobile/services/OrderService.dart';
import 'package:flutter/material.dart';

class PartnerOrderModal extends StatefulWidget {
  final PartnerOrderModel pom;
  final Function refresh;
  final Function closed;
  PartnerOrderModal(this.pom, this.refresh, this.closed);

  @override
  _PartnerOrderModalState createState() => _PartnerOrderModalState(pom);
}

class _PartnerOrderModalState extends State<PartnerOrderModal> {
  late Timer _timer;
  int _time = 60;
  bool close = false;

  final orderService = new OrderService();

  void timer() {
    const oneSec = const Duration(seconds: 1);
    _timer = new Timer.periodic(oneSec, (Timer timer) {
      if (!mounted) {
        timer.cancel();
        return;
      }

      if (ModalRoute.of(context)!.isCurrent) {
        this.setState(() {});
        if (_time == 0) {
          close = true;
          timer.cancel();
          return;
        }
        _time--;
      }
    });
  }

  _PartnerOrderModalState(PartnerOrderModel pom) {
    close = false;
    timer();
  }

  void pressedButtonRecusar(BuildContext context) async {
    _timer.cancel();
    widget.pom.status = OrderStatus.REFUSED;
    await orderService.updateOrder(widget.pom);

    if (!close) {
      widget.refresh();
    }
    widget.closed();
    Navigator.pop(context);
  }

  void pressedButtonAceitar(BuildContext context) async {
    _timer.cancel();
    widget.pom.status = OrderStatus.ACCEPTED;
    await orderService.updateOrder(widget.pom);

    widget.refresh();
    widget.closed();
    Navigator.pop(context);
  }

  @override
  Widget build(BuildContext context) {
    if (close) {
      pressedButtonRecusar(context);
    }
    return Container(
      height: MediaQuery.of(context).size.height * 0.5,
      child: Center(
        child: Column(children: [
          Container(
            width: 150,
            child: Image.asset("lib/assets/icons/drone_icon.png"),
          ),
          Padding(
            padding: EdgeInsets.fromLTRB(0, 0, 0, 20),
            child: Text(
              "Novo Pedido",
              style: TextStyle(fontSize: 24, fontWeight: FontWeight.bold),
            ),
          ),
          Expanded(
              child: SingleChildScrollView(
            child: getColumn(),
          )),
          Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              ElevatedButton(
                  onPressed: () => {pressedButtonRecusar(context)},
                  child: Text("Rejeitar (" + _time.toString() + ")",
                      style: TextStyle(fontSize: 20)),
                  style: ElevatedButton.styleFrom(minimumSize: Size(92, 36))),
              Padding(padding: EdgeInsets.fromLTRB(50, 0, 0, 0)),
              ElevatedButton(
                  onPressed: () => {pressedButtonAceitar(context)},
                  child: Text(
                    "Aceitar",
                    style: TextStyle(fontSize: 20),
                  ),
                  style: ElevatedButton.styleFrom(
                      primary: Colors.blue, minimumSize: Size(92, 36)))
            ],
          )
        ]),
      ),
    );
  }

  Column getColumn() {
    List<Widget> list = [];
    list.add(Padding(
      padding: EdgeInsets.fromLTRB(0, 0, 0, 10),
      child: Text('Local de entrega: ${widget.pom.deliveryAddress.description}',
          style: TextStyle(fontSize: 16)),
    ));
    list.add(Padding(
      padding: EdgeInsets.fromLTRB(0, 0, 0, 10),
      child: Text('Número: ${widget.pom.deliveryAddress.number}',
          style: TextStyle(fontSize: 16)),
    ));
    list.add(Padding(
      padding: EdgeInsets.fromLTRB(0, 0, 0, 10),
      child:
          Text('Peso: ${calculateWeight()} g', style: TextStyle(fontSize: 16)),
    ));
    list.add(Padding(
      padding: EdgeInsets.fromLTRB(0, 0, 0, 10),
      child: Text('Valor do pedido: R\$ ${widget.pom.orderPrice}',
          style: TextStyle(fontSize: 16)),
    ));
    list.add(Padding(
      padding: EdgeInsets.fromLTRB(0, 0, 0, 10),
      child: Text('Valor da entrega: R\$ ${widget.pom.deliveryPrice}',
          style: TextStyle(fontSize: 16)),
    ));
    list.add(Padding(
      padding: EdgeInsets.fromLTRB(0, 0, 0, 10),
      child: Text(
          'Total: R\$ ${widget.pom.deliveryPrice + widget.pom.orderPrice}',
          style: TextStyle(fontSize: 16)),
    ));
    list.add(Padding(
      padding: EdgeInsets.fromLTRB(0, 0, 0, 10),
      child: Text('Produtos: ', style: TextStyle(fontSize: 16)),
    ));

    for (var i = 0; i < widget.pom.items.length; i++) {
      list.add(
        Padding(
            padding: EdgeInsets.fromLTRB(0, 0, 0, 10),
            child: Text(
              widget.pom.items[i].quantity.toString() +
                  "x  " +
                  widget.pom.items[i].product.name,
              style: TextStyle(fontSize: 16),
            )),
      );
    }

    return new Column(children: list);
  }

  double calculateWeight() {
    double weight = 0;
    for (int i = 0; i < widget.pom.items.length; i++) {
      OrderItemModel o = widget.pom.items[i];
      weight += (o.quantity * o.product.weight);
    }
    return weight;
  }
}
