import 'package:apus_drones_mobile/models/OrderModel.dart';
import 'package:apus_drones_mobile/models/TripModel.dart';
import 'package:apus_drones_mobile/widgets/Price.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';

class TripCard extends StatelessWidget {
  final TripModel model;
  final void Function()? onTap;
  const TripCard({Key? key, required this.model, this.onTap}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Card(
      elevation: 3,
      margin: EdgeInsets.symmetric(horizontal: 15, vertical: 7),
      child: InkWell(
        onTap: onTap,
        child: Padding(
          padding: const EdgeInsets.symmetric(vertical: 10, horizontal: 3),
          child: ListTile(
            title: Text(model.order.partner.name),
            leading: Container(
              height: 60,
              width: 60,
              decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(4),
                  image: new DecorationImage(
                    fit: BoxFit.cover,
                    alignment: FractionalOffset.center,
                    image: new NetworkImage(model.order.partner.avatarUrl),
                  )),
            ),
            trailing: Column(
              children: [
                Text(DateFormat('dd/MM/yyyy').format(model.createdAt)),
                Price(price: model.order.orderPrice)
              ],
            ),
            subtitle: Padding(
              padding: const EdgeInsets.only(top: 8.0),
              child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: this
                      .model
                      .order
                      .items
                      .map((item) => Text(
                          item.quantity.toString() + "x " + item.product.name))
                      .toList()),
            ),
          ),
        ),
      ),
    );
  }
}
