import 'package:apus_drones_mobile/constants/OrderStatus.dart';
import 'package:apus_drones_mobile/models/CustomerModel.dart';
import 'package:apus_drones_mobile/models/OrderModel.dart';
import 'package:apus_drones_mobile/models/PartnerModel.dart';
import 'package:apus_drones_mobile/models/ProductModel.dart';
import 'package:apus_drones_mobile/services/OrderService.dart';
import 'package:apus_drones_mobile/widgets/AddToCart.dart';
import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/annotations.dart';
import 'package:mockito/mockito.dart';

import 'AddToCart_test.mocks.dart';

ProductModel product = ProductModel(
    id: 1,
    name: 'Carregador',
    description: 'de iPhone',
    price: 100,
    status: "ativo",
    weight: 20,
    createdAt: DateTime.now(),
    quantity: 1,
    partner: PartnerModel(
      id: 1,
      avatarUrl: '',
      name: '',
    ));

OrderModel orderFromOtherPartner = OrderModel(
    id: 1,
    partner: PartnerModel(id: 2, name: 'iPlace', avatarUrl: ''),
    customer:
        CustomerModel(id: 1, name: '', email: '', cpfCnpj: '', avatarUrl: ''),
    status: OrderStatus.IN_CART,
    createdAt: DateTime.now(),
    items: [],
    deliveryPrice: 10,
    orderPrice: 100);

@GenerateMocks([OrderService])
void main() {
  testWidgets('AddToCart should display text', (WidgetTester tester) async {
    await tester.pumpWidget(render());
    validateProductPricing();
  });

  testWidgets('AddToCart should display error message for different partner',
      (WidgetTester tester) async {
    final orderService = MockOrderService();
    when(orderService.myCart()).thenAnswer((_) async => orderFromOtherPartner);
    await tester.pumpWidget(render(service: orderService));
    await tester.tap(find.byType(ElevatedButton));
    await tester.pumpAndSettle(const Duration(seconds: 1));
    validateErrorMessage(
        "Não é possível adicionar produtos de mais um parceiro ao carrinho. Já há produtos do parceiro ${orderFromOtherPartner.partner.name}.");
  });

  testWidgets('AddToCart should add item to cart', (WidgetTester tester) async {
    final orderService = MockOrderService();
    when(orderService.myCart()).thenAnswer((_) async => null);
    await tester.pumpWidget(render(service: orderService));
    await tester.tap(find.byType(ElevatedButton));
    verify(orderService.addToCart(any)).called(1);
  });

  testWidgets('AddToCart should show weight alert',
      (WidgetTester tester) async {
    final orderService = MockOrderService();
    when(orderService.myCart()).thenAnswer((_) async => null);
    when(orderService.addToCart(any)).thenThrow((_) async => null);
    await tester.pumpWidget(render(service: orderService));
    await tester.tap(find.byType(ElevatedButton));
    await tester.pumpAndSettle(const Duration(seconds: 1));
    verify(orderService.addToCart(any)).called(1);
    validateOkButton(
        "Não é possível adicionar mais itens ao seu carrinho, pois o peso ultrapassa o limite.");
  });
}

Widget render({OrderService? service}) {
  return MaterialApp(
    home: AddToCart(
      product: product,
      service: service ?? MockOrderService(),
    ),
  );
}

void validateProductPricing() {
  final buttonTextFinder = find.text('ADICIONAR ');
  expect(buttonTextFinder, findsOneWidget);
}

void validateErrorMessage(String message) {
  final errorTitleFinder = find.text('Erro');
  final errorMessageFinder = find.text(message);
  expect(errorTitleFinder, findsOneWidget);
  expect(errorMessageFinder, findsOneWidget);
}

void validateOkButton(String message) {
  final okButtonFinder = find.text('Ok');
  final contentFinder = find.text(message);
  expect(okButtonFinder, findsOneWidget);
  expect(contentFinder, findsOneWidget);
}
